@@ -1,6 +1,8 @@
 BART_MAX_MEM_MB_DEFAULT = 1100 #1.1GB is the most a 32bit machine can give without throwing an error or crashing
 BART_NUM_CORES_DEFAULT = 1 #Stay conservative as a default
 
+#' @rdname bartMachine
+#' @export
 ##build a BART model
 build_bart_machine = function(X = NULL, y = NULL, Xy = NULL, 
 		num_trees = 50, #found many times to not get better after this value... so let it be the default, it's faster too 
@@ -31,8 +33,43 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 		flush_indices_to_save_RAM = TRUE,
 		serialize = FALSE,
 		seed = NULL,
+		use_xoshiro = FALSE,
 		verbose = TRUE){
 
+  # Validate arguments
+  assert_data_frame(X, null.ok = TRUE)
+  if (!is.null(y)) assert_atomic_vector(y)
+  assert_data_frame(Xy, null.ok = TRUE)
+  assert_int(num_trees, lower = 1)
+  assert_int(num_burn_in, lower = 0)
+  assert_int(num_iterations_after_burn_in, lower = 1)
+  assert_number(alpha, lower = 0, upper = 1)
+  assert_number(beta, lower = 0)
+  assert_number(k, lower = .Machine$double.eps)
+  assert_number(q, lower = .Machine$double.eps, upper = 1 - .Machine$double.eps)
+  assert_number(nu, lower = .Machine$double.eps)
+  assert_number(prob_rule_class, lower = 0, upper = 1)
+  assert_numeric(mh_prob_steps, len = 3, lower = .Machine$double.eps)
+  assert_flag(debug_log)
+  assert_flag(run_in_sample)
+  assert_choice(s_sq_y, c("mse", "var"))
+  assert_number(sig_sq_est, lower = .Machine$double.eps, null.ok = TRUE)
+  assert_flag(print_tree_illustrations)
+  assert_numeric(cov_prior_vec, lower = .Machine$double.eps, null.ok = TRUE)
+  assert_list(interaction_constraints, null.ok = TRUE)
+  assert_flag(use_missing_data)
+  assert_int(num_rand_samps_in_library, lower = 1)
+  assert_flag(use_missing_data_dummies_as_covars)
+  assert_flag(replace_missing_data_with_x_j_bar)
+  assert_flag(impute_missingness_with_rf_impute)
+  assert_flag(impute_missingness_with_x_j_bar_for_lm)
+  assert_flag(mem_cache_for_speed)
+  assert_flag(flush_indices_to_save_RAM)
+  assert_flag(serialize)
+  assert_int(seed, null.ok = TRUE)
+  assert_flag(use_xoshiro)
+  assert_flag(verbose)
+
 	if (verbose){
 		cat("bartMachine initializing with", num_trees, "trees...\n")	
 	}	
@@ -78,7 +115,9 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 	y_levels = levels(y)
 	if (inherits(y, "numeric") || inherits(y, "integer")){ #if y is numeric, then it's a regression problem
 		if (inherits(y, "integer")){
-			cat("Warning: The response y is integer, bartMachine will run regression.\n")
+			if (verbose){
+				cat("Warning: The response y is integer, bartMachine will run regression.\n")
+			}
 		}
 		#java expects doubles, not ints, so we need to cast this now to avoid errors later
 		if (inherits(y, "integer")){
@@ -147,8 +186,8 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 			predictor_colnums_with_missingness = names(which(colSums(is.na(X)) > 0))
 			
 			rf_imputations_for_missing = rfImpute(X, y)
-			rf_imputations_for_missing = rf_imputations_for_missing[, 2 : ncol(rf_imputations_for_missing)]
-			rf_imputations_for_missing = rf_imputations_for_missing[, predictor_colnums_with_missingness]
+			rf_imputations_for_missing = rf_imputations_for_missing[, 2 : ncol(rf_imputations_for_missing), drop = FALSE]
+			rf_imputations_for_missing = rf_imputations_for_missing[, predictor_colnums_with_missingness, drop = FALSE]
 		}
 		colnames(rf_imputations_for_missing) = paste(colnames(rf_imputations_for_missing), "_imp", sep = "")
 		if (verbose){
@@ -202,30 +241,34 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 			stop("specified parameter \"interaction_constraints\" must be a list")
 		} else if (length(interaction_constraints) == 0){
 			stop("interaction_constraints list cannot be empty")
-		}		
+		}
 		
+		col_names = colnames(X)
 		for (a in 1 : length(interaction_constraints)){
 			vars_a = interaction_constraints[[a]]
+			vars_a_converted = integer(length(vars_a))
 			#check if the constraint components are valid features
 			for (b in 1 : length(vars_a)){
-				var = vars_a[b]
-				if ((inherits(var, "numeric") | inherits(var, "integer")) & !(var %in% (1 : p))){
-					stop(paste("Element", var, "in interaction_constraints vector number", a, "is numeric but not one of 1, ...,", p, "where", p, "is the number of columns in X."))
-				}
+				var = vars_a[[b]]
 				if (inherits(var, "factor")){
 					var = as.character(var)
 				}
-				if (inherits(var, "character")  & !(var %in% colnames(X))){
-					stop(paste("Element", var, "in interaction_constraints vector number", a, "is a string but not one of the column names of X."))
-				}
-				#force all it be integers and begin index at zero
-				if (inherits(var, "integer") | inherits(var, "numeric")){
-					vars_a[b] = var - 1
-				} else if (inherits(var, "character")){
-					vars_a[b] = which(colnames(X) == var) - 1
+				if (inherits(var, "character")){
+					idx = match(var, col_names)
+					if (is.na(idx)){
+						stop(paste("Element", var, "in interaction_constraints vector number", a, "is a string but not one of the column names of X."))
+					}
+					vars_a_converted[b] = idx - 1
+				} else if (inherits(var, "numeric") || inherits(var, "integer")){
+					if (var %% 1 != 0 || var < 1 || var > p){
+						stop(paste("Element", var, "in interaction_constraints vector number", a, "is numeric but not one of 1, ...,", p, "where", p, "is the number of columns in X."))
+					}
+					vars_a_converted[b] = as.integer(var) - 1
+				} else {
+					stop(paste("Element", var, "in interaction_constraints vector number", a, "has unsupported type."))
 				}
 			}
-			interaction_constraints[[a]] = as.integer(vars_a)
+			interaction_constraints[[a]] = vars_a_converted
 		}
 	}
 	#print("interaction_constraints:"); print(interaction_constraints)
@@ -249,7 +292,9 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 	}
 	#set whether we want there to be tree illustrations
 	if (print_tree_illustrations){
-		cat("warning: printing tree illustrations is excruciatingly slow.\n")
+		if (verbose){
+			cat("warning: printing tree illustrations is excruciatingly slow.\n")
+		}
 		.jcall(java_bart_machine, "V", "printTreeIllustations")
 	}
 	
@@ -321,14 +366,12 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 	.jcall(java_bart_machine, "V", "setVerbose", verbose)
 	.jcall(java_bart_machine, "V", "setMemCacheForSpeed", mem_cache_for_speed)
 	.jcall(java_bart_machine, "V", "setFlushIndicesToSaveRAM", flush_indices_to_save_RAM)
+	.jcall(java_bart_machine, "V", "setUseXoshiro", use_xoshiro)
 	
 #	cat("seed", seed, "\n")
 	if (!is.null(seed)){
 		#set the seed in Java
 		.jcall(java_bart_machine, "V", "setSeed", as.integer(seed))
-		if (num_cores > 1){
-			warning("Setting the seed when using parallelization does not result in deterministic output.\nIf you need deterministic output, you must run \"set_bart_machine_num_cores(1)\" and then build the BART model with the set seed.")
-		}
 	}
 	
 	#now we need to set random samples
@@ -440,6 +483,7 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 			flush_indices_to_save_RAM = flush_indices_to_save_RAM,
 			debug_log = debug_log,
 			seed = seed,
+			use_xoshiro = use_xoshiro,
 			num_rand_samps_in_library = num_rand_samps_in_library
 	)
 	#if the user used a cov prior vec, pass it back
@@ -500,9 +544,13 @@ build_bart_machine = function(X = NULL, y = NULL, Xy = NULL,
 	
 	#Let's serialize the object if the user wishes
 	if (serialize){
-		cat("serializing in order to be saved for future R sessions...")
+		if (verbose){
+			cat("serializing in order to be saved for future R sessions...")
+		}
 		.jcache(bart_machine$java_bart_machine)
-		cat("done\n")
+		if (verbose){
+			cat("done\n")
+		}
 	}
 	
 	#use R's S3 object orientation
@@ -556,19 +604,35 @@ bart_machine_duplicate = function(bart_machine, X = NULL, y = NULL, cov_prior_ve
 		impute_missingness_with_x_j_bar_for_lm = bart_machine$impute_missingness_with_x_j_bar_for_lm,
 		mem_cache_for_speed = bart_machine$mem_cache_for_speed,
 		serialize = FALSE, #we do not want to waste CPU time here since these are created internally by us
+		use_xoshiro = bart_machine$use_xoshiro,
 		verbose = verbose)
 }
 
 #build a BART-cv model
+#' @rdname bartMachineCV
+#' @export
 build_bart_machine_cv = function(X = NULL, y = NULL, Xy = NULL, 
 		num_tree_cvs = c(50, 200),
 		k_cvs = c(2, 3, 5),
 		nu_q_cvs = NULL,
 		k_folds = 5, 
 		folds_vec = NULL, 
-		verbose = FALSE,
+		use_xoshiro = FALSE,
+		verbose = TRUE,
 		...){
 	
+  # Validate arguments
+  assert_data_frame(X, null.ok = TRUE)
+  if (!is.null(y)) assert_atomic_vector(y)
+  assert_data_frame(Xy, null.ok = TRUE)
+  assert_integerish(num_tree_cvs, lower = 1, min.len = 1)
+  assert_numeric(k_cvs, lower = 0, min.len = 1)
+  assert_list(nu_q_cvs, null.ok = TRUE)
+  assert_count(k_folds, positive = TRUE)
+  assert_integerish(folds_vec, null.ok = TRUE)
+  assert_flag(use_xoshiro)
+  assert_flag(verbose)
+
 	if ((is.null(X) && is.null(Xy)) || is.null(y) && is.null(Xy)){
 		stop("You need to give bartMachine a training set either by specifying X and y or by specifying a matrix Xy which contains the response named \"y.\"\n")
 	} else if (!is.null(X) && !is.null(y) && !is.null(Xy)){
@@ -639,10 +703,12 @@ build_bart_machine_cv = function(X = NULL, y = NULL, Xy = NULL,
 		for (nu_q in nu_q_cvs){
 			for (num_trees in num_tree_cvs){
 				
-				if (pred_type == "regression"){
-					cat(paste("  bartMachine CV try: k:", k, "nu, q:", paste(as.numeric(nu_q), collapse = ", "), "m:", num_trees, "\n"))	
-				} else {
-					cat(paste("  bartMachine CV try: k:", k, "m:", num_trees, "\n"))
+				if (verbose){
+					if (pred_type == "regression"){
+						cat(paste("  bartMachine CV try: k:", k, "nu, q:", paste(as.numeric(nu_q), collapse = ", "), "m:", num_trees, "\n"))	
+					} else {
+						cat(paste("  bartMachine CV try: k:", k, "m:", num_trees, "\n"))
+					}					
 				}
 				
 				k_fold_results = k_fold_cv(X, y, 
@@ -673,53 +739,57 @@ build_bart_machine_cv = function(X = NULL, y = NULL, Xy = NULL,
 			}
 		}
 	}
-	if (pred_type == "regression"){
-		cat(paste("  bartMachine CV win: k:", min_rmse_k, "nu, q:", paste(as.numeric(min_rmse_nu_q), collapse = ", "), "m:", min_rmse_num_tree, "\n"))
-	} else {
-		cat(paste("  bartMachine CV win: k:", min_rmse_k, "m:", min_rmse_num_tree, "\n"))
+	if (verbose){
+		if (pred_type == "regression"){
+			cat(paste("  bartMachine CV win: k:", min_rmse_k, "nu, q:", paste(as.numeric(min_rmse_nu_q), collapse = ", "), "m:", min_rmse_num_tree, "\n"))
+		} else {
+			cat(paste("  bartMachine CV win: k:", min_rmse_k, "m:", min_rmse_num_tree, "\n"))
+		}
 	}
 	#now that we've found the best settings, return that bart machine. It would be faster to have kept this around, but doing it this way saves RAM for speed.
 	bart_machine_cv = build_bart_machine(X, y,
 			num_trees = min_rmse_num_tree,
 			k = min_rmse_k,
 			nu = min_rmse_nu_q[1],
-			q = min_rmse_nu_q[2], ...)
+			q = min_rmse_nu_q[2], 
+			verbose = verbose,
+			use_xoshiro = use_xoshiro,
+			...)
 	
 	#give the user some cv_stats ordered by the best (ie lowest) oosrmse
 	cv_stats = cv_stats[order(cv_stats[, "oos_error"]), ]
 	cv_stats[, 6] = (cv_stats[, 5] - cv_stats[1, 5]) / cv_stats[1, 5] * 100
 	bart_machine_cv$cv_stats = cv_stats
-  	bart_machine_cv$folds = folds_vec
+  bart_machine_cv$folds = folds_vec
 	bart_machine_cv
 }
 
 ##private function for filling in missing data with averages for cont. vars and modes for cat. vars
 imputeMatrixByXbarjContinuousOrModalForBinary = function(X_with_missing, X_for_calculating_avgs){
-	for (i in 1 : nrow(X_with_missing)){
-		for (j in 1 : ncol(X_with_missing)){
-			if (is.na(X_with_missing[i, j])){
-				#mode for factors, otherwise average
-				if (inherits(X_with_missing[, j], "factor")){
-					X_with_missing[i, j] = names(which.max(table(X_for_calculating_avgs[, j])))
-				} else {
-					X_with_missing[i, j] = mean(X_for_calculating_avgs[, j], na.rm = TRUE)
-				}
-			}
+	replacements = vector("list", ncol(X_with_missing))
+	for (j in seq_len(ncol(X_with_missing))){
+		#mode for factors, otherwise average
+		if (inherits(X_with_missing[, j], "factor")){
+			replacements[[j]] = names(which.max(table(X_for_calculating_avgs[, j])))
+		} else {
+			replacements[[j]] = mean(X_for_calculating_avgs[, j], na.rm = TRUE)
+		}
+	}
+	for (j in seq_len(ncol(X_with_missing))){
+		missing_idx = is.na(X_with_missing[, j])
+		if (any(missing_idx)){
+			X_with_missing[missing_idx, j] = replacements[[j]]
 		}
 	}
 	#now we have to go through and drop columns that are all NaN's if need be
-	bad_cols = c()
-	for (j in colnames(X_with_missing)){
-		if (sum(is.nan(X_with_missing[, j])) == nrow(X_with_missing)){
-			bad_cols = c(bad_cols, j)
+	bad_cols = vapply(X_with_missing, function(col){
+		if (!is.numeric(col)){
+			return(FALSE)
 		}
-	}
-	for (j in bad_cols){
-		X_with_missing[, j] = NULL
+		sum(is.nan(col)) == nrow(X_with_missing)
+	}, logical(1))
+	if (any(bad_cols)){
+		X_with_missing = X_with_missing[, !bad_cols, drop = FALSE]
 	}
 	X_with_missing
 }
-
-destroy_bart_machine = function(bart_machine){
-	warning("the method \"destroy_bart_machine\" does not do anything anymore")
-}
\ No newline at end of file
